import argparse
import sys
from rich.console import Console

console = Console()

def print_help():
    console.print("Usage:", style="bold")
    console.print("  python -m occt [", end="")
    console.print("--options", style="yellow", end="")
    console.print("] ", end="")
    console.print("<command>", style="cyan")
    console.print()

    console.print("Options:", style="bold")
    console.print("  -c, ", style="yellow", end=""); console.print("--compile", style="yellow", end="")
    console.print("         Pass flag for Compiled Code output file. [Auto Detect OS and Programming, and save output file]")

    console.print("  -o, ", style="yellow", end=""); console.print("--output", style="yellow", end="")
    console.print("          Pass flag only used with -c flag, for output file custom name.")

    console.print("  -i, ", style="yellow", end=""); console.print("--interpreted", style="yellow", end="")
    console.print("     Pass flag for interpret Code output only.")

    console.print("  -a, ", style="yellow", end=""); console.print("--args", style="yellow", end="")
    console.print("            After this flag all value consider as command line arguments.")

    console.print("  -d, ", style="yellow", end=""); console.print("--debug", style="yellow", end="")
    console.print("           Print debug information if needed.")

    console.print("  -v, ", style="yellow", end=""); console.print("--version", style="yellow", end="")
    console.print("         Print version information.")

    console.print("  -h, ", style="yellow", end=""); console.print("--help", style="yellow", end="")
    console.print("            Print the help information.")

    console.print("  --clear", style="yellow", end="")
    console.print("               Clear the terminal window. (default: true)")

    console.print("  --no-clear", style="yellow", end="")
    console.print("            Don't clear the terminal window.")
    console.print()

    console.print("Commands:", style="bold")
    console.print("  help", style="cyan", end="")
    console.print("                  Print the help information.")
    console.print()

    console.print("Example:", style="bold")
    console.print("  python -m occt ", end=""); console.print("-c", style="yellow", end=""); console.print(" ./main.cpp ", end=""); console.print("-o", style="yellow", end=""); console.print(" main")
    console.print("  python -m occt ", end=""); console.print("-i", style="yellow", end=""); console.print(" ./app.js")
    console.print()



import requests
import sys
import json
from pathlib import Path
from yaspin import yaspin

BASE_SERVER_URL = "https://occt.in"

def handle_error(error, name="API Call fail"):
    print(f"\n[ERROR] {name}: {error}")
    sys.exit(1)


def uploading_file_compile(flags):
    with yaspin(text="Uploading file and processing...", color="yellow") as spinner:
        try:
            file_path = Path(flags.get("compile"))
            if not file_path.exists():
                handle_error("File not found")

            files = {"file": (file_path.name, open(file_path, "rb"))}
            data = {}

            if flags.get("outputFile"):
                data["outputName"] = flags["outputFile"]

            if flags.get("args"):
                data["args"] = json.dumps(flags["args"])

            response = requests.post(f"{BASE_SERVER_URL}/compile", files=files, data=data)

            if response.status_code != 200:
                handle_error(response.text)

            resp_data = response.json()
            download_url = f"{BASE_SERVER_URL}{resp_data['download']}"
            output_file = resp_data["download"].replace("/download/", "")

            download = requests.get(download_url, stream=True)
            with open(output_file, "wb") as f:
                for chunk in download.iter_content(1024):
                    f.write(chunk)

            spinner.ok("✅")
            print(f"Compiled file: {output_file}")

        except Exception as e:
            spinner.fail("💥")
            handle_error(e)


def uploading_file_interprate(flags, cli=None):
    cli = cli or {}
    with yaspin(text="Uploading file and processing...", color="yellow") as spinner:
        try:
            file_path = Path(flags.get("interpreted"))
            if not file_path.exists():
                handle_error("File not found")

            files = {"file": (file_path.name, open(file_path, "rb"))}
            data = {}

            if flags.get("outputFile"):
                data["outputName"] = flags["outputFile"]

            if flags.get("args"):
                args_val = flags["args"]
                if cli.get("input") and cli.get("flags", {}).get("args"):
                    args_val += " " + " ".join(cli["input"])
                data["args"] = json.dumps(str(args_val))

            response = requests.post(f"{BASE_SERVER_URL}/interprate", files=files, data=data)

            if response.status_code != 200:
                handle_error(response.text)

            resp_data = response.json()
            spinner.ok("✅")
            print(f"\n---\n{resp_data['output']}---")

        except Exception as e:
            spinner.fail("💥")
            handle_error(e)



# occt/__init__.py

# Read package.json file for description and version
import json
from .states import uploading_file_compile, uploading_file_interprate
from pathlib import Path

# file path to package.json
PACKAGE_JSON_PATH = "package.json"
package_info = {}
with open(PACKAGE_JSON_PATH, "r") as f:
    package_info = json.load(f)
    description = package_info.get("description", "No description available.")
    version = package_info.get("version", "0.0.0")


# ANSI color codes
BLACK = "\033[30m"
WHITE = "\033[97m"
CYAN = "\033[96m"
YELLOW_BG = "\033[43m"
DIM = "\033[2m"
RESET = "\033[0m"


# CLI info
title = " OCCT "
version = f"v{version}"
tagline = "Learn more, innovate more!!"
description = description


def show_banner():
    # Clear the terminal screen
    print("\033c", end="")
    # Title with yellow background and black text
    print(f"{YELLOW_BG}{BLACK}{title}{RESET} {version} {DIM}{tagline}{RESET}")
    # Description in cyan
    print(f"{DIM}{description}{RESET}")
    print()

def show_extra_options(args, console, print_help):
    if args.version:
        console.print(f"Version: {version}", style="cyan")
    elif args.compile:
        flags_compile = {
            "compile": args.compile,
            "outputFile": args.output,
            "args": args.args
        }
        uploading_file_compile(flags_compile)
    elif args.interpreted:
        flags_interpreted = {
            "interpreted": args.interpreted,
            "args": args.args
        }
        uploading_file_interprate(flags_interpreted, cli={"input": ["extra"], "flags": {"args": True}})
    else:
        print_help()

def run():
    show_banner()



# occt/__main__.py
import sys
from . import run, show_extra_options
import os
import argparse
import sys
from rich.console import Console
console = Console()
from .help import print_help

def main():
    run()
    if len(sys.argv) < 2 or sys.argv[1] in ("-h", "--help", "help"):
        print_help()
        return
    
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("-c", "--compile", type=str)
    parser.add_argument("-o", "--output", type=str)
    parser.add_argument("-i", "--interpreted", type=str)
    parser.add_argument("-a", "--args", nargs=argparse.REMAINDER)
    parser.add_argument("-d", "--debug", action="store_true")
    parser.add_argument("-v", "--version", action="store_true")
    parser.add_argument("--clear", action="store_true", default=True)
    parser.add_argument("--no-clear", action="store_true")

    args = parser.parse_args(sys.argv[1:])

    show_extra_options(args, console, print_help)
        

if __name__ == "__main__":
    main()
